@@ -8,8 +8,8 @@ use vars qw($VERSION $revision);
 #------------------------------------------------------------------
 # global stuff
 #------------------------------------------------------------------
-$VERSION = '0.07';
-$revision = '$Id: Rcs.pm,v 1.14 1998/07/23 01:00:23 freter Exp $';
+$VERSION = '0.08';
+$revision = '$Id: Rcs.pm,v 1.14.1.2 1998/09/11 07:38:48 freter Exp $';
 my $Dir_Sep = ($^O eq 'MSWin32') ? '\\' : '/';
 my $Exe_Ext = ($^O eq 'MSWin32') ? '.exe' : '';
 my $Rcs_Bin_Dir = '/usr/local/bin';
@@ -214,36 +214,56 @@ sub comments {
 
 #------------------------------------------------------------------
 # daterev
-# Returns a revision which was current at a specified date/time.
-# 0 is returned if all revisions are newer than the date 
-# specified. This usually means the file did not exist on that
-# date.
-# This takes 6 parameters, year (4 digit year), month (1-12), day
+#
+# Returns revisions which were created before a specified date.
+#
+# Method takes one or six arguments.
+#
+# If one argument, then argument is date number.
+#
+# If six arguments, then year (4 digit year), month (1-12), day
 # of month (1-31), hour (0-23), minute (0-59) and second (0-59).
 #------------------------------------------------------------------
 sub daterev {
-    my $self = shift;
-    my($year, $mon, $mday, $hour, $min, $sec) = @_;
 
-    # ensure date has all the elements
-    if(@_ != 6) {
-        croak "daterev must have 6 element date/time (year, month, day, hour, min, sec)";
+    my $self = shift;
+    my $target_time;
+
+    # validate arguments
+    unless (@_ == 1 or @_ == 6) {
+        croak "daterev must have either 1 or 6 arguments";
     }
 
-    if($year !~ /^\d{4}$/) {
-        croak "year (1st param) must be 4 digit number";
+    # string date passed
+    if (@_ == 6) {
+        my($year, $mon, $mday, $hour, $min, $sec) = @_;
+
+        if($year !~ /^\d{4}$/) {
+            croak "year (1st param) must be 4 digit number";
+        }
+
+        $mon--;        # convert to 0-11 range
+        $target_time = timegm($sec, $min, $hour, $mday, $mon, $year);
+    }
+
+    # system date passed
+    else {
+        $target_time = shift;
+
+        if ($target_time !~ /^\d+$/) {
+            croak "system date must be an integer";
+        }
     }
 
     if (not defined $self->{DATE}) {
         _parse_rcs_header($self);
     }
 
-    $mon--;        # convert to 0-11 range
-    my $target_time = timegm($sec, $min, $hour, $mday, $mon, $year);
-    my @revisions;
+    my @revisions = ();
     my %dates;
-
     my %dates_hash = %{$self->{DATE}};
+
+    my $revision;
     foreach $revision (keys %dates_hash) {
         my $date = $dates_hash{$revision};
         $dates{$date}{$revision} = 1;
@@ -693,43 +713,52 @@ sub _parse_rcs_body {
     my $rcs_file = $rcsdir . $Dir_Sep . $file . ${ $self->{"_ARCEXT"} };
 
     # parse RCS archive file
-    open RCS_FILE, $rcs_file
-        or croak "Unable to open $rcs_file";
+    open RCS_FILE, $rcs_file or croak "Unable to open $rcs_file";
 
     # skip header info and get description
     DESC: while (<RCS_FILE>) {
         if (/^desc$/) {
+            $comments{0} = '';
+            $_ = <RCS_FILE>;      # read first line
+            s/^\@//;              # remove leading '@'
             while (1) {
-                $_ = <RCS_FILE>;
-                (chomp $comments{0} and last DESC)
-                    if /^\@$/;
-                s/^\@//;
+                last DESC if /^\@$/;
+                s/\@\@/\@/g;      # RCS replaces single '@' with '@@'
                 $comments{0} .= $_;
+                $_ = <RCS_FILE>;
             }
         }
     }
 
-    # parse body
+    # parse revision comments
     my $revision;
     REVISION: while (<RCS_FILE>) {
         if (/^[\d\.]+$/) {
-            chomp(my $revision = $_);
-            my $next = <RCS_FILE>;
-            if ($next =~ /^log$/) {
+            chomp($revision = $_);
+            $_ = <RCS_FILE>;
+            if (/^log$/) {
+                $comments{$revision} = '';
+                $_ = <RCS_FILE>;  # read first line
+                s/^\@//;          # remove leading '@'
                 while (1) {
-                    $_ = <RCS_FILE>;
-                    (chomp $comments{$revision} and next REVISION)
-                        if /^\@$/;
-                    s/^\@//;
+                    next REVISION if /^\@$/;
+                    s/\@\@/\@/g;  # RCS replaces single '@' with '@@'
                     $comments{$revision} .= $_;
+                    $_ = <RCS_FILE>;
                 }
             }
         }
     }
+
+    # loop through 'text' section to avoid capturing bogus info
+    continue {
+        if (/^text$/) {  # 'text' tag should always be there, but check anyway
+            while (<RCS_FILE>) {last if /^\@$/}
+        }
+    }
+
     close RCS_FILE;
-
     $self->{COMMENTS} = \%comments;
-
 }
 
 #------------------------------------------------------------------
@@ -1037,14 +1066,20 @@ defined. If the symbol was not defined against any version of this file, 0 is
 returned.
 
     # gets revision that has 'MY_SYMBOL' defined against it
-    $rev = symrev('MY_SYMBOL');
+    $rev = $obj->symrev('MY_SYMBOL');
 
-The B<daterev> method returns a revision which was current at a specified
-date/time. If all revisions are newer than the specified date/time, i.e. the
-file did not exist then, 0 is returned.
+The B<daterev> method returns revisions which were created before a specified
+date.  Method may take one or six arguments.  If one arguments is passed, then
+the argument is a date number.  If six arguments are passed, then they represent
+a date string.
 
-    # gets revision that was active on 25th June 1998 16:45:30
-    $rev = daterev(1998, 6, 25, 16, 45, 30);
+    # one argument, date number
+    # gets revisions created before Sun Sep  6 22:23:47 1998
+    @revs = $obj->daterev(841436420);
+
+    # six argument
+    # gets revisions created before 25th June 1998 16:45:30
+    @revs = $obj->daterev(1998, 6, 25, 16, 45, 30);
 
 The B<comments> method returns a hash of revision comments, keyed on revision.
 A key value of 0 returns the description.
@@ -1293,13 +1328,8 @@ Craig Freter, E<lt>F<craig@freter.com>E<gt>
 
 David Green, E<lt>F<greendjf@cvhp152.gpt.co.uk>E<gt>
 
-    David Green contributed the B<dates> method.
-
 Jamie O'Shaughnessy, E<lt>F<jamie@thanatar.demon.co.uk>E<gt>
 
-    Contributed NT port.
-    Contributed methods B<daterev>, B<symrev>, and B<symbols>.
-
 =head1 COPYRIGHT
 
 Copyright (C) 1997,1998 Craig Freter.  All rights reserved.
